        program cafi
c Calculations of Acoustic Fluctuations due to Internal Waves
c code defined and prepared by Galina Rovner and Stanley Flatte'
c Version made public: August 1998
c
c Reads parameter file from re-directed standard input.
c (Read "README" file for run instructions.);
c Calls subroutines to follow rays through the ocean.
c As ray is followed, other subroutines are called that
c calculate the effects of internal waves.  Upon reaching
c the defined receiver range, results are written on several
c output files.
c Three input ocean profiles are required: sound speed, buoyancy
c frequency, and sound-speed variance due to internal waves.
c
c Michael G. Brown's WKBJ program (adapted for CAFI input and output)
c is used as a part of the system if the appropriate option is
c chosen in the parameter file.
c
c -read "User's Guide to CAFI" for descriptions of the parameter file,
c ocean profile files, and the output files.
c
        integer hcount,timeit
        logical nofloq,yfluct,ylong,yray2,error
        logical reflct
        character*30 sfile,bfile,flfile,itime,idate
        character*30 lgfile,wkbjou,mufile
        double precision wron,s2rw
      real zdl(1000),tdl(1000),ttdl(1000),s2bdl(1000),bfdl(1000)
     &,pwtdl(1000),psqidl(1000),pdwt(1000),pdidl(1000),
     &qkv(1000),qkvsq(1000)
      real lat,n0b,sangl(300),rangl(300),jstar,msqmax,na,njinv,mj
c next lines are added for plot.
      real kv,kvsq,nu,nuprim,nusq,lambda,logphi,lamphi,lmphsq,nupsq
      real a1phsq,lamba1,a2nusq,lamba2,a3kvsq,lamba3,lamh10,
     &lamh11,lambh1,lamh20,lamh21,lambh2,lamh30,lamh31,lambh3
      common/maxmin/rmax1,zmax,rzmax,zmin,rzmin,rtmax,ztmax,angmax
      common/kv/kvsq,kv
      common/tau/tau1,tau0sq,tau0
      common/gam/gamj,gam
      common/nu/nusq,nu,nupsq,nuprim,phovnu
      common/lphi/phisq,phi,logphi,lambda,lamphi,lmphsq
      common/valid/a1phsq,lamba1,a2nusq,lamba2,a3kvsq,lamba3,lamh10,
     &lamh11,lambh1,lamh20,lamh21,lambh2,lamh30,lamh31,lambh3
      common/recevr/timetr,tl,ata,angler,zrc
      common/loops/nloops,itmin,rfrac
      common/muparm/ssm,dym
c end of extra lines for plot.
      common/iounit/iuinp,iuout
       common/itime/itime,idate
      common/mpparm/sdepth,angle,h,hcount,rfinal,rdepth,freq,lat
     &,nofloq
        common/parm/q0,q0inv,q0sq,range,n0b,wi,winv
      common/rayst/zdl,tdl,ttdl,s2bdl,bfdl,pwtdl,psqidl,pdwt,pdidl
     &,qkv,qkvsq
        common/ssparm/ss,dy
        common/bfparm/sss,dyy
        common/flparm/na,jstar,zeta,gamad,msqmax,mj,njinv,avj
        common/names/sfile,bfile
         common/muname/mufile
        common/const/pi
        common/sext/zx0,up0,rdpi,rdpinv
        common/rms/th,thh
	common/sr/s1r,s2r,wron,s2rw
        common/horiz/hseps,hsepr,wkhsq
        common/option/yfluct,ylong,yray2
        common/surfac/reflct
        common/rayplt/plrang,plstep
c  common blocks for connection with wkbj
        common/wkbcaf/rdep0,rfin0,sdep0,freqw,ngrp1,ngrp2
        common/wkbnam/wkbjou
        common/wkbstp/deltaa,deltaz
c end common blocks for wkbj
        common/case/nflag
         save
        data pi/3.14159265359/
        data iuinp,iuout/5,7/
        data hseps/0.0/,hsepr/1000.0/
c
          call hour(itime)
          call date(idate)
1002      format(' start of run ', a10,2x,a10)
          nofloq=.false.
          con=180.*180./(2.*pi)
          con=alog10(con)
c read the name of the sound speed profile
        read *, sfile
c read the name of the buoyancy frequency profile
        read *, bfile
c read the name of the fluctuations profile
        read *, mufile
c read the name of the 'short' output file
        read *, flfile
c read the name of the 'long output' file
        read *, lgfile
c read source depth, range, receiver depth, central frequency
        read *, sdep0,rfin0,rdep0,freq
c horizontal separation parameters
        read *, hseps,hsepr
c average latitude,N0B, average gamma adiabatic 
        read *, lat,n0b,gamad
c integrations step,number of steps in one storred in array parameter
        read *, h0,hcnt0
c reference buoyancy frequency,charcter. mode number,zeta_0,max(<musq>)
        read *, na,jstar,zeta,msqmax
c smoothing parameters for smthn.f
        read *, ss,dy,sss,dyy
c rays choice
        read *, nflag
c to do or not fluctuations calculations, long output, parabolic correction
        read *, yfluct,ylong,yray2
        if(ylong) open(iuout,file=lgfile)
c ray plot parameters
        read *, ipl1,nirapl,ipstep,plrang,plstep,rang1
        if(nflag) 101,103,105
c number of rays to do calculations for
101     read *, nira
c parameters for wkbj
        read *, ngrp1,ngrp2,deltaa,deltaz
c min. relative amplitude value 
        read *, rampmn
c wkbj output file name
        read *, wkbjou
        freqw=freq
        call wkbj(nira,sangl,rampmn,rangl)
        go to 107
c number of rays and their angle at source
103     read *, nira,(sangl(i),i=1,nira)
        go to 107
c number of rays, first ray's angle, step in angle for each next ray
105     read *, nira,angle0,delang
107     continue
        rewind (iuout)
        npl=1
          if(ylong)write(iuout,1002) idate,itime
        if(nirapl.gt.99) open(11,file='rayplots200')
        open(4,file=flfile)
        rewind 4
        rdest=50.
        rdpi=rdest*1000./(2.*pi)
        rdpinv = 1./rdpi
c loop over the number of rays, do all the calculations
        do 20 ira=1,nira
c set up initial conditions for the ray
        error=.false.
        reflct=.false.
        jtime00=timeit()
	rdepth = rdep0
        rfinal = rfin0
	sdepth = sdep0
	h = h0
	hcount =hcnt0
        if(nofloq) print *, 'nofloq is true'
c angle is the ray angle at source
        if(nflag.gt.0) then
         angle=angle0+delang*(ira-1)
        else
        angle=sangl(ira)
        endif
c call ssinit in sspeed.f to open sound speed profile
        call ssinit
c call bfinit in bvfreq.f to open buoyancy frequency profile
        call bfinit
c call flinit in fluct.f to open fluctuations profile or calculate
        call flinit
c calculate function for Lp 
        call lpinit
c initiate the ray, then rayini.f calls other subroutines
c to step the ray along and do all the calculations
        call rayini(error)
        if (error) then
        if (ipl1.eq.ira) ipl1=ipl1+1
         go to 20
        endif
        if(nflag.ne.-1) angler=ata
c zmin is the upper turning point depth
c nloops is number of loops 
c rmax1 is the double-loop length (m)
c do adjustments for a surface-reflected rays (zmin=0.0)
c nutp is number of upper turning points
        if (reflct) then
        nloops=nloops*2
        rmax1=rmax1/2.0
        if(rfrac.ge.rmax1) then
        nloops=nloops+1
        rfrac=rfrac-rmax1
        endif
        if(rzmin.gt.rmax1) rzmin=rzmin-rmax1
        if(rzmax.gt.rmax1) rzmax=rzmax-rmax1
        endif
        nutp=nloops
        rayid=nloops*2.
        if(rfrac.ge.rzmin) then
         nutp=nutp+1
         rayid=rayid+1.
        endif
        if(rfrac.ge.rzmax) rayid=rayid+1.
        rayid=sign(rayid,angle)
       idray=int(rayid)
c c0 is min(sound speed) 
        call ssc0(c0)
c convert tau* to milliseconds
c tau* are fluctuations parameters-see 'User's Guide to CAFI'
        tau=1000.*phi/(q0*c0)
        tau1=1000.*tau1
        tau0=1000.*tau0
        print *,'ira=',ira
c phovnu=phi*t0, where t0=1/nu(see timesc.f), 1./nuprim =ti, t0,ti in hours
c we write out frequency-independant values now, phi*t0 and phi*ti. Sep2,94,glr
       write(4,801) angle,zmin,rzmin,zmax,rzmax,rmax1,angmax,
     & zrc,ata,idray,timetr,tl,tau,tau0,tau1,phovnu,phi/nuprim,
     & lamba1*freq,thh,th
          call flush(4)
c call uplall or userpl to calculate data for rays parameters plot(s)
c  as a function(s) of range-
c if such an option is chosen
        if(nirapl.gt.99) then
c for a large number of rays to plot only ray coordinates:depth vs. range
c all in one output file uplall.f is used
        call uplall(rang1)
        go to 2002
        endif
        if(nirapl.ne.0) then
        if(npl.le.nirapl) then
        if(ira.eq.ipl1) then 
c for a few rays (or, as many as 99)-to plot ray geometry
c parameters-you can choose, which-you have to edit the subroutine userpl.f
c it writes the ray parameters for each ray in separate file ray*,
c where * is the ray number (for rays chosen in parameter file) 
         call userpl(npl)
        npl=npl+1
        ipl1=ipl1+ipstep
         endif
         endif
         endif
2002     if(ylong) then
        jtime01=timeit()
        timeall=float(jtime01-jtime00)/60.
c how long did it take to do the calculations for one ray?-print it out
c in 'long output'
        write(iuout,9000) timeall
        write(iuout,2000)
        endif
 20     continue
        if(ylong) then
        call hour(itime)
        call date(idate)
        write(iuout,1001)idate,itime
 1000   endfile iuout
        endif
800     format(1x,f7.3,1x,f7.1,1x,f10.1,1x,f7.1,1x,f10.1,f10.1,1x,
     & f7.3,1x,f7.1,1x,f7.3,i7,f10.4,1x,f8.2)
801     format(1x,f7.3,1x,f7.1,1x,f10.1,1x,f7.1,1x,f10.1,f10.1,1x,
     & f7.3,1x,f7.1,1x,f7.3,i7,f10.4,1x,f8.2,f8.3,2(1x,f13.3),
     & 2f8.4,2f9.4,1x,g12.5)
802     format(1x,f7.3,5(1x,f15.2))
2000    format('\f')
9000    format(1x,'cpu sec elapsed to do all ray calculations =',g14.5)
 1001   format(' end of run ',a10,2x,a10)
        close (4)
        close (9)
        stop
        end
