      function w0avg(wi,wl,n)
c
c purpose
c  calculate 1 / ( wl**2 * Mw ) where Mw is defined by eqn. 37
c  in R. Esswein and S. M. Flatte, J. Acoust. Soc. Amer., 70(5), 1981
c value returned
c  w0avg      1 / ( wl**2 * Mw )
c argument list
c  wi     in  the inertial frequency
c  wl     in  lower limit of internal wave frequencies
c  n      in  the buoyancy frequency
c method
c  use the closed form expression for the integral
c version 1.0 15 Dec 81
c version 1.1 25 Mar 82
c  return zero if n less than wl
c by R. F. Esswein, Ocean Acoustics Group, Physics, U. C. Santa Cruz
c
          real n
          double precision wi2,wl2,n2,r,s1,s2
          if (n.gt.wl) go to 50
          w0avg=0.
          return
   50     n2=n*n
          wi2=wi*wi
          if ((wl-wi).gt.0.00015) go to 100
          w0avg=(1./wi2-1./n2)/2.
          return
  100     wl2=wl*wl
          r=sqrt((n2-wi2)*(n2-wl2))
          s1=n2*(wi2+wl2) + 2.*(wi*wl*r - wi2*wl2)
          s2=n2*(wl2-wi2)
          w0avg= (wl2-wi2)*dlog(s1/s2)/(4.*wi*wl*wl2)+r/(2.*wl2*n2)
          return
          end
