      function w2avg(wi,wl,n)
c
c purpose
c  calculate { w**2 } / ( wl**2 * Mw ) where { w**2 } is defined
c  in the Appendix ( top of p. 1396 ) to
c  R. Esswein and S. M. Flatte, J. Acoust. Soc. Amer., 70(5), 1981
c value returned
c  w2avg      { w**2 } / ( wl**2 * Mw )
c argument list
c value returned
c  w0avg      1 / ( wl**2 * Mw )
c argument list
c  wi     in  the inertial frequency
c  wl     in  lower limit of internal wave frequencies
c  n      in  the buoyancy frequency
c method
c  use the closed form expression for the integral
c version 1.0 15 Dec 81
c version 1.1 25 Mar 82
c  return zero if n less than wl
c by R. F. Esswein, Ocean Acoustics Group, Physics, U. C. Santa Cruz
c
          real n
         double precision wi2,wl2,n2,s1,s2,r
c          double precision wi2,wl2,n2,r
          if (n.gt.wl) go to 50
          w2avg=0.
          return
   50     if ((wl-wi).gt.0.00015) go to 100
          w2avg=alog(n/wi)
          return
  100     wi2=wi*wi
          wl2=wl*wl
          n2=n*n
          r=sqrt((n2-wi2)*(n2-wl2))
          s1=n2*(wi2+wl2) - 2.*(wi*wl*r + wi2*wl2)
          s2=n2*(wl2-wi2)
          w2avg=(wi*dlog(s1/s2)/wl
     *           +dlog((2.*(r+n2)-wi2-wl2)/(wl2-wi2)))/2.
          return
          end
