       subroutine wkbj(nira,sangl,rampmn,rangl)
c
c  this program computes the wkbj green's function (inpulse response) for
c  ocean acoustic propagation (i.e. evaluates equation 9 in michael g.
c  brown, 'application of the wkbj green's function to acoustic propagation
c  in horizontally stratified oceans', jasa).  convolution with
c  a source after the inpulse response has been evaluated is an option.
c
c     last modified: 02 mar 1983, peter worcester                       030283
c                                                                       021883
      dimension rindx(1601),geom(300,8),plotid(12)                       021883
      dimension zdepth(1601),cdepth(1601),qz(3,301),fz(3,301),ff(10000)
      dimension a(10000),zt(10000),zb(10000)
      dimension ampl(10000),theta(10000),p(3,10000)
      dimension iseq(300)
      dimension sangl(300),rangl(300)
c
      logical lcsf,ltheta,lampf,lsec,undef
      character*30 sfile,bfile
      character*30 wkbjou
c
      common/one/zt,zb,zs,zr,rindxs,rindxr,cmin,dpth
      common/two/rindx,dz
      common/seven/tstart,tend,nt,pltlen,rng
      common/sgram/sgm
      common/tsgram/tsgm
      common/ssparm/ss,dy
      common/iounit/iuinp,iuout
      common/names/sfile,bfile
c  new common blocks-for connection with caffi and using 'paramoap' instead 
c  of 'wkbjin'
      common/wkbcaf/rdep0,rfin0,sdep0,freqw,ngrp1,ngrp2
      common/wkbnam/wkbjou
      common/wkbstp/deltaa,deltaz
      common/zparm/zmin,zmax
c
c  read in profile identifier, ocean depth, source and receiver depths, range,
c  start time, end time, time window, carrier frequency, bandwidth ,number
c  of points in impulse response, plotlength (inches), ray parameter increment,
c  and minimum ray parameter -note units are m, m/s, s, 1/s, etc
c  if tstart and tend are both non-zero twndo is ignored.  if tstart and tend
c  are both set equal to zero twndo must be non-zero. in this case a (usually
c  good) guess at tstart and tend, where tend-tstart=twndo, is made.
c  nt may be set equal to zero- in this case nt is selected so that there are
c  eight samples per cycle of the carrier frequency
      open (4,file=wkbjou)
      rewind 4
      zs=sdep0
      zr=rdep0
      rng=rfin0
      freq=freqw
c                                                                       031283
c     set nt=0 so that the program will always use 8 samples/cycle      031283
c                                                                       031283
      nt=0                                                              031283
c      deltaa=0.000125
c      deltaa=0.00025
      rinmin=0.94
c
c  read in logical variables for:  1)spherical earth
c  correction, 2)amplitude correction for scattering
      lsec=.false.
      lampf=.false.
c
c  read in depths (m) and corresponding sound speeds (m/s) 
c  from sfile (compile program with sspeed.f to apply sph. earth correct.,
c  -loa to include ssint,sslist,sspeed)
      call ssinit
c      call sslist
c  interpolate using cubic spline to get sound speeds at evenly spaced
c  depths (separated by dz) from top to bottom
      dz=deltaz
      if(dz.eq.0.0)    dz=10.0
      dpth=zmax
      npts=int(dpth/dz)+1
      do 5 i=1,npts
      s=dz*float(i-1)
      zdepth(i)=s
      call sspeed(s,rindx(i),dc,d2ur,undef)
5     continue
c
c  apply spherical earth correction
      if(.not.lsec)go to 4
      re=6.371e06
      eps=zs/re
      zs=zs*(1.+eps/2.+(eps**2)/3.)
      eps=zr/re
      zr=zr*(1.+eps/2.+(eps**2)/3.)
c
c
c  read in ray identifiers
4     continue
      nray1=1
      nray2=8
      twndo=2.4
c      normally nray1=1 and nray2=12 to include all ray types
c     we use nray2=8 which means we are not including bottom
c     reflected rays. Feb.09,1988.glr.
c
c
c     write(6,1019)                                                     031283
c1019 format(' geometric ray travel times and identifiers')             031283
c
c  find cmin,naxis,zaxis,rmin
      cmin=rindx(1)
      naxis=1
      do 8 i=2,npts
      if(rindx(i).gt.cmin)go to 8
c cmin must be non-zero
      if(rindx(i).ne.0.0) go to 7
      rindx(i)=rindx(i-1)
      print *,'rindx(i)=0.! i=',i,'rindx(i-1)=',rindx(i-1)
      if(rindx(i).gt.cmin)go to 8
7      cmin=rindx(i)
      naxis=i
8     continue
      zaxis=float(naxis-1)*dz
      rmin=rindx(1)
      do 9 i=1,naxis
9     if(rindx(i).gt.rmin)rmin=rindx(i)
c
c      print *,'rmin=',rmin,'naxis=',naxis,'rindx(npts)=',rindx(npts)
c      print *,'cmin=',cmin
      if(tstart.ne.0..or.tend.ne.0.)go to 3
      t=rng/cmin*.996
      ii=int(t)
      if((t-float(ii)).gt..5)ii=ii+1
      t=float(ii)
      tstart=t-twndo/2.
      tend=tstart+twndo
c      print *,'tstart=',tstart,'tend=',tend
c    next line '8' means 8 samples/cycle
3     if(nt.eq.0) nt=8*int((tend-tstart)*freq)+1
      ntmax=10000                                                       031283
c      if(nt.gt.ntmax) stop                                         031283
c nt>ntmax means the frequency is high and we need to use smaller twndo
      if(nt.gt.ntmax) then
       twndo=float(ntmax-1)/(8.*freq)    
      tstart=t-twndo/2.
      tend=tstart+twndo
      nt=8*int((tend-tstart)*freq)+1
c
      endif
      write(4,*) sfile
2000  format(1h1,12a4)
      write(4,2001)zaxis,dpth,cmin
2001  format(14h axial depth =,f7.2,2h m,4x,14hbottom depth =,f7.2,2h m,
     .4x,13haxial speed =,f7.2,6h m/sec)
      rngkm=.001*rng
      write(4,2002)zs,zr,rngkm
2002  format(15h source depth =,f7.2,2h m,4x,16hreceiver depth =,f7.2,
     .2h m,4x,7hrange =,f9.3,3h km)
      write(4,2008)tstart,tend,nt,deltaa
2008  format(9h tstart =,f8.2,14h sec    tend =,f8.2,10h sec  nt =,i5,
     .8h pts    ,10h  deltaa =,f7.5)
      write(4,2009)ngrp1,ngrp2,nray1,nray2
2009  format(i2,9h.le.m.le.,i2,2x,i1,9h.le.j.le.,i2)
      if(lcsf)write(4,2010)freq,bw
2010  format(' center frequency =',f6.1,' hz    bandwidth =',f6.1,' hz')
      write(4,1009)dz
1009  format(' sound speed in m/sec at ',f6.2,' m intervals starting ',
     .'at the surface')
c      write(4,1010)(rindx(i),i=1,npts)
1010  format(10f8.2)
c
c  calculate index of refraction
      do 10 i=1,npts
10    rindx(i)=cmin/rindx(i)
      rmin=cmin/rmin
c
c  calculate a's
      nbra=0
      nbrsra=0
      nzr=int(zr/dz)+1
      nzs=int(zs/dz)+1
      dzs=zs/dz-float(int(zs/dz))
      dzr=zr/dz-float(int(zr/dz))
      rindxs=rindx(nzs)+dzs*(rindx(nzs+1)-rindx(nzs))
      rindxr=rindx(nzr)+dzr*(rindx(nzr+1)-rindx(nzr))
      ns=int((rindxs-rmin)/deltaa)
      if(ns.lt.0)ns=0
      nr=int((rindxr-rmin)/deltaa)
      if(nr.lt.0)nr=0
      rimin=amin1(rindxs,rindxr)
      rimax=amax1(rindxs,rindxr)
      if(nray2.le.8)go to 13
      nbra=int((rindx(npts)-rinmin)/deltaa)
c  rinmin is minimun bottom reflected ray parameter
      if(nbra.gt.1)go to 11
      nbra=0
      go to 13
11    del=(rindx(npts)-rinmin)/float(nbra-1)
      do 12 i=1,nbra
12    a(i)=rinmin+float(i-1)*del
      a(nbra)=rindx(npts)
13    if(nray2.le.4)go to 16
      rminum=amin1(rimin,rmin)
      nsra=int((rminum-rindx(npts))/deltaa)
c  rmin is the minimum rindx above the axis
c  rminum is min(rindxr,rindxs,rmin)
      if(nsra.gt.1)go to 14
      nsra=0
      nbrsra=nbra
      go to 16
14    del=(rminum-rindx(npts))/float(nsra-1)
      nbra1=nbra+1
      nbrsra=nbra+nsra
      do 15 i=nbra1,nbrsra
15    a(i)=rindx(npts)+float(i-nbra1)*del
      a(nbrsra)=rminum
16    nra=int((rimin-rmin)/deltaa)
c  rimin is min(rindxs,rindxr)
      if(nra.gt.1)go to 17
      nra=0
      na=nbrsra
      nao=nbrsra
      if(lampf)go to 18
      go to 21
17    del=(rimin-rmin)/float(nra-1)
      na=nbrsra+nra
      nao=na
      if(.not.lampf)go to 19
18    cc=1.3e-04
      cc=.8*cc
c     13.=5.6*2.3 so that pdf has sqrt(e<(a-ao)**2>)=5.6e-05, as predicted
      rngd2=rng/2.
      phi=cc*sqrt(rngd2)
      x=sqrt(1-rimin**2)*sin(phi)/deltaa
      nc=int(x)
      if((x-float(nc)).gt..5)nc=nc+1
      na=nao+nc
19    nbrsr1=nbrsra+1
      do 20 i=nbrsr1,na
20    a(i)=rmin+float(i-nbrsr1)*del
      if(.not.lampf)a(na)=rimin
21    continue
1020  format('nao =',i3,4x,'na =',i3)
c      do 22 i=1,na
c      i24=i
c22    if(a(i).ge..9801)go to 18
c
c  calculate upper and lower turning points
      do 25 i=1,na
      zt(i)=0.
      k=naxis
23    if(rindx(k).ge.a(i))go to 24
      zt(i)=dz*(float(k-1)+(a(i)-rindx(k))/(rindx(k+1)-rindx(k)))
      go to 25
24    if(k.eq.1)go to 25
      k=k-1
      go to 23
25    continue
      do 35 i=1,na
      zb(i)=dpth
      do 30 k=naxis,npts
      if(rindx(k).ge.a(i))go to 30
      zb(i)=dz*(float(k-2)+(a(i)-rindx(k-1))/(rindx(k)-rindx(k-1)))
      go to 35
30    continue
35    continue
c
c  calculate phase integrals
      do 40 i=1,na
      p(1,i)=0.
      p(2,i)=0.
      p(3,i)=0.
      if(a(i).ge.1.)go to 40
      p(1,i)=phint(a(i),zs,zb(i))
      p(2,i)=phint(a(i),zt(i),zs)
      if(zr.gt.zs)p(3,i)=-phint(a(i),zs,zr)
      if(zr.eq.zs)p(3,i)=0.
      if(zr.lt.zs)p(3,i)=phint(a(i),zr,zs)
40    continue
c
c      write(4,1003)
1003  format(75h1     cos(axis   upper     lower     ampl         zs
     .     zt          zr)
c      write(4,1005)
1005  format(75h       angle)    turn m    turn m                 zb
     .     zs          zs)
c
c  calculate amplitudes
      pairy=.109*cmin/freq
      f=0.
      if(rindxr.lt.rindxs)f=1.
      do 50 i=1,nao
      ii=i
      if((p(1,i)+f*p(3,i)).le.pairy)go to 51
50    continue
51    del=(p(1,ii-1)+f*p(3,ii-1)-pairy)/
     .    (p(1,ii-1)+f*p(3,ii-1)-p(1,ii)-f*p(3,ii))
      aairy=a(ii-1)+del*(a(ii)-a(ii-1))
c      ampmax=.95/sqrt(sqrt(rimin**2-aairy**2))
      ampmax=5.85
      nrr=nbrsra+nr
      do 55 i=1,nrr
      if(i.eq.nrr)go to 55
      ff(i)=1./sqrt(sqrt(rindxr**2-a(i)**2))
      if(ff(i).gt.ampmax)ff(i)=ampmax
c      if(ff(i).gt.ampmax)ampmax=ff(i)
55    continue
      ff(nrr)=ampmax
      nss=nbrsra+ns
      do 60 i=1,nss
      if(i.eq.nss)go to 60
      ampl(i)=1./sqrt(sqrt(rindxs**2-a(i)**2))
      if(ampl(i).gt.ampmax)ampl(i)=ampmax
c      if(ampl(i).gt.ampmax)ampmax=ampl(i)
60    continue
      ampl(nss)=ampmax
      if(lampf)go to 65
      do 61 i=1,nao
      ff(i)=ff(i)*ampl(i)
61    ampl(i)=0.
      go to 70
65    call ampfix(ff,nrr,na,nc)
      call ampfix(ampl,nss,na,nc)
      do 66 i=1,na
      ff(i)=ff(i)*ampl(i)
66    ampl(i)=0.
70    do 71 i=1,na
      gg=ff(i)*sqrt(a(i))
c71    write(4,1002)i,a(i),zt(i),zb(i),gg,p(1,i),p(2,i),p(3,i)
71    continue
1002  format(i4,f9.4,1x,3f9.2,3f12.2)
c
c      write(4,2000)(plotid(i),i=1,12)
c      write(4,2001)zaxis,dpth,cmin
c      write(4,2002)zs,zr,rngkm
c      write(4,2003)
2003  format(//19h geometric arrivals)
c      write(4,2004)
2004  format(87h travel     source  receiver cos(axs iden-  sign  upper
     .  lower   turning  relative m,j)
c      write(4,2005)
2005  format(83h  time      angle    angle    angle  tifier (r')  turn m
     .  turn m  speed m/s ampltd )
c
c  compute sofargrams
c  initialize sgm, geom(geometric travel time & amplitude  array),
c  and nga(the number of geometrical arrivals)
      do 77 i=1,300
      do 77 j=1,8
77    geom(i,j)=0.
      nga=0
c  sum up ray groups
      do 100 m=ngrp1,ngrp2
c  each group consists of 12 different ray types
      do 95 j=nray1,nray2
      if(m.eq.0.and.(j.ne.2.and.j.ne.6.and.j.ne.10))go to 95
      call reflco(m,j,ir,sgn)
      if(j.gt.4)go to 81
      istart=nbrsr1
      iend=na
      go to 83
81    if(j.gt.8)go to 82
      istart=nbra1
      iend=nbrsra
      go to 83
82    istart=1
      iend=nbra
83    continue
c      if(j.eq.6.or.j.eq.8.or.j.eq.10.or.j.eq.12)go to 99
c      if(j.eq.2.or.j.eq.4)istart=i24
      do 85 i=istart,iend
      ampl(i)=-ff(i)*sgn*sqrt(a(i))
c  calculate theta's
      si=1.
      if(zr.gt.zs)si=-1.
c      phi=arcos(a(i))*57.29578
c      if(j.eq.5)ampl(i)=ampl(i)*exp(-.02*(phi+si*5.)**2)
c      if(j.eq.6)ampl(i)=ampl(i)*exp(-.02*(phi-si*5.)**2)
c      if(j.eq.7)ampl(i)=ampl(i)*exp(-.02*(phi+5.)**2)
c      if(j.eq.8)ampl(i)=ampl(i)*exp(-.02*(phi-5.)**2)
      if(j.eq.1.or.j.eq.5.or.j.eq.9)
     .   theta(i)=float(2*(m-1))*(p(1,i)+p(2,i))+si*p(3,i)
      if(j.eq.2.or.j.eq.6.or.j.eq.10)
     .   theta(i)=float(2*m)*(p(1,i)+p(2,i))-si*p(3,i)
      if(j.eq.3.or.j.eq.7.or.j.eq.11)
     .   theta(i)=float(2*(m-1))*p(1,i)+float(2*m)*p(2,i)-p(3,i)
      if(j.eq.4.or.j.eq.8.or.j.eq.12)
     .   theta(i)=float(2*m)*p(1,i)+float(2*(m-1))*p(2,i)+p(3,i)
      theta(i)=(theta(i)+a(i)*rng)/cmin
85    continue
c      write(4,900)(theta(i),i=istart,iend)
900   format(10f8.3)
c
c  compute real and imaginary parts of sgm
      call gmray(a,theta,ampl,istart,iend,m,j,geom,nga)
95    continue
100   continue
c
c  compute tsgm from real and imaginary parts of sgm
c      print *,'enter sorttt'
c      if(nira.gt.nga) nira=nga
c      if(nira.eq.0) nira=nga
c commented out 2 'nira' lines above. added new, next
      nira=nga
      call sorta(geom(1,8),nga,iseq,nira,iamp)
c      print *,'iamp=',iamp,'nira=',nira
      call sorttt(geom(1,1),nga,iseq)
c      print *,'exit sorttt'
      write(4,2020)
 2020 format(//' sorted by travel time')
      write(4,2003)
      write(4,3004)
3004  format(78h   travel      source  receiver  iden-   upper   lower  
     .  turning     relative)
      write(4,3005)
3005  format(78h    time       angle    angle    tifier  turn m  turn m 
     .  speed m/s    ampltd )
      iangl=0
       iangl1=0
c      print *,'enter 5000 loop'
      do 5000 i=1,nga
      k=iseq(i)
      nturn=ifix(geom(k,4))
      write(4,5001) (geom(k,j),j=1,3),nturn,(geom(k,j),j=5,8)
5001  format(f11.4,1x,2(f7.3,2x),i6,1x,2f9.0,f10.2,3x,f6.3)
      if(iangl.ge.nira) go to 5000
      if(geom(k,8).ge.geom(iamp,8)) then
      iangl=iangl+1
      if(geom(k,8).lt.rampmn) go to 5000
      iangl1=iangl1+1
       sangl(iangl1)=geom(k,2)
       rangl(iangl1)=geom(k,3)
      endif
 5000 continue
      nira=iangl1
c      print *,'wkbj,returning to caffi,ss=',ss,'dy=',dy
c
      close(1)
      close(4)
      return
      end
