      subroutine fluct(n,c,dc,tangnt,musq,lp,lv,undef)
c     this subroutine calculates certain functions of sound speed
c     fluctuations due to internal waves at a given ray element
c     depth and orientation
c       arguments
c         n       the buoyancy freq at depth of the ray element
c         tangnt the slope of the ray element w.r.t. horizontal
c       returned
c         musq    the variance of sound speed fluctuations
c         lp      the correlation length of the fluctuations
c                 parallel to the ray element
c         lv      the vertical correlation length of the fluctuations
c     computes musq at a given depth by cubic spline
c     interpolation of smoothed experimental data.
c     the cubic spline coefficients are contained in array scoef.
c     they are input from a file in the directory at runtime.
c Lp computed by lpap.f (call lpofr()) using empirical formula approximate
c numerical lp(r,rho) calculations which use F(k) function.(Read 'User's
c Guide to CAFI', or S.Flatte "Path-integral expression for fluctuations
c in acoustic transmission in the ocean waveguide",1997).
c For numerical Lp compile with lpc.f instead of lpap.f
c musq correction included, to make musq=0.0 at surface and reduce
c musq values in upper 300m ze1=55.0 musq=musq*(1.-exp(-depth/ze1))
c
      real zmdata(1000),mudata(1000),smdata(1000),dyam(1000),
     & c1m(1000),c2m(1000),c3m(1000),nconv
          character*10 flname,legnd(6,3),form(6)
         character*30 mufile
	logical nofloq,undef
        logical ylong,yfluct,yray2
      real zdl(1000),tdl(1000),ttdl(1000),s2bdl(1000),bfdl(1000),
     *pwtdl(1000),psqidl(1000),pdwt(1000),pdidl(1000),
     *qkv(1000),qkvsq(1000),lat,n0b
        integer hcount
	real na,jstar,msqmax,mjinv,mj,njinv,n0br,nar,lp,lv,musq,n,n0
	double precision x,a1,a2
	common /iounit/ iuinp,iuout
        common /muname/ mufile
	common /flname/ flname
       common/depthf/depth
      common/mpparm/sdepth,angle,h,hcount,rfinal,rdepth,freq,lat,nofloq
	common/parm/q0,q0inv,q0sq,range,n0b,wi,winv
	common/rayst/zdl,tdl,ttdl,s2bdl,bfdl,pwtdl,psqidl,pdwt,pdidl,
     *qkv,qkvsq
	common/flparm/na,jstar,zeta,gamad,msqmax,mj,njinv,avj
	common/const/pi
        common/option/yfluct,ylong,yray2
        common/flloc/n0,nar,zeta2,const1,const2
c new common/flloc/ Oct.10,94
          common/muparm/ssm,dym
        common/zmudat/iz,zmdata,smdata,dyam,c1m,c2m,c3m,s0,nmdata
c
        common/xrotst/xro
        save
	data na,jstar /3.,3./
	data zeta,gamad /7.3,1.1e-5 /
	data msqmax /0.01/
	data gamma0/.577215664902/
	data flname / 10hintwave2  /
c
          data iz/1/
          data zmmin,zmmax,ssm,dym/1000.,0.,-1.,-1./
          data lfn / 9 /
c
          if (depth-zmdata(iz)) 20,35,10
   10     if (iz.ge.nmdata) go to 51
          if (depth.le.zmdata(iz+1)) go to 35
          iz=iz+1
          go to 10
   20     iz=iz-1
          if (iz.eq.0) go to 45
          if (depth.ge.zmdata(iz)) go to 35
          go to 20
   35     h=depth-zmdata(iz)
          musq=smdata(iz)+h*(c1m(iz)+h*(c2m(iz)+h*c3m(iz)))
          q=nar/n
c          musq=zeta2*q*(dc/c-gamad)**2
          if (musq.ge.msqmax) then
           print *,'musq>msqmax,depth,musq',depth,musq
            go to 100
            endif
          lv=const1/n
          if (n*n*(1.-tangnt*tangnt).le.wi*wi) go to 50
c          x=n*tangnt/wi
          x=n*sin(atan(tangnt))/wi
            xro=abs(sngl(x))
            call lpofr(lp,x,c,dc)
c          if (abs(x).lt.0.0001) go to 40
c          a1=x*x+1.
c          a2=sqrt(a1)
c          f1=(1./a1)+((0.5*x*x)/(a1*a2))*dlog((a2+1.)/(a2-1.))
c          lp=const2*f1/wi
          undef=.false.
          return
c   40     lp=const2/wi
c          undef=.false.
c          return
   50     lp=0.
          undef=.false.
          return
  100     undef=.true.
          return
   45     iz=nmdata/2
   51     undef=.true.
          return
c
      entry flinit
c
c
          n0br=2.*pi*n0b
          nar=2.*pi*na
          zeta2=zeta*zeta
          njinv=(pi/tanh(pi*jstar)-1./jstar)/(2.*jstar)
          mjinv=(jstar**(-2))*(gamma0+alog(jstar)+jstar**(-2)/12.+
     *           jstar**(-4)/120.)
          mj=1./mjinv
          avj=mjinv/njinv
          const1=n0br*sqrt(mjinv)/pi
          const2=avj*4.*n0br/(pi*pi)
c ze1 is for musq correction
         ze1=55.0
c     finds coefficients for a cubic spline interpolation of
c     experimental data.
c
c     read data in 
c
          open(lfn,file=mufile)
          rewind lfn
          read (lfn,1000) legnd,flname
          read (lfn,*) stemp
          read (lfn,*) dytemp
          read (lfn,*) nmdata
          read (lfn,*) z1
          read (lfn,*) z2
          read (lfn,*) zconv
          read (lfn,*) nconv
          read (lfn,1100) form
          if (ssm.lt.0.) ssm=stemp
 1000     format(3(6a10/),a10)
 1100     format(6a10)
          If (dym.lt.0.) dym=dytemp
c          if (zmin.lt.zmax) go to 450
          zmmax=z2
          zmmin=z1
  450     continue
          zmmin=-zmmax
          nmdat0=nmdata
          nmdata=2*nmdata
          j0=nmdat0
         do 300 i=1,nmdat0
              read (lfn,form) depth,speed
c correction to musq!! ze1=55.0 May 9,97
                 adepth=abs(depth)
      if(adepth.lt.600.0) speed=speed*(1.-exp(-adepth/ze1))
              zmdata(i)=zconv*depth
              mudata(i)=nconv*speed
              dyam(i)=dym
300           continue
            close(lfn)
c     calculate smoothed data points
c we don't use smoothing, usually: the fluctuations mudata are very small,
c so we use dyam=0.0. for the "close to the surface" depths we correct by
c  exponential: the surface value of musq must be 0.0.
c  we call smthn() to find the cubic spline coefficients for points on the ray
c and we want to use the method consistent with sspeed.f and bvfreq.f
           if(zmdata(1).eq.0.0) then
           nmdata=nmdata-1
           j0=nmdat0-1
           endif
           nmdatn=nmdat0
          call smthn(1,nmdatn,zmdata,mudata,dyam,ssm,smdata,
     & c1m,c2m,c3m)
               iz=nmdat0
               smdata(nmdata)=smdata(nmdat0)
               zmdata(nmdata)=zmdata(nmdat0)
               c1m(nmdata)=c1m(nmdat0)
               c2m(nmdata)=c2m(nmdat0)
               c3m(nmdata)=c3m(nmdat0)
               smdata(nmdat0)=mudata(1)
               zmdata(nmdat0)=zmdata(1)
               c1m(nmdat0)=c1m(1)
               c2m(nmdat0)=c2m(1)
               c3m(nmdat0)=c3m(1)
c              do 350 i=1,nmdat0
              do 350 i=1,j0
               j=j0+i
              smdata(j)=smdata(i)
              zmdata(j)=zmdata(i)
              c1m(j)=c1m(i)
              c2m(j)=c2m(i)
              c3m(j)=c3m(i)
350         continue
               i1=0
              if(zmdata(1).eq.0.0) i1=1
444        do 500 i=1,j0
              j=nmdata-i+1
              zmdata(i)=-zmdata(j)
              smdata(i)=smdata(j)
              c1m(i)=c1m(j)
              c2m(i)=c2m(j)
              c3m(i)=c3m(j)
500        continue
               smdata(1)=mudata(nmdat0)
          zmdata(1)=zmmin
c test 
c           open(12,file='fltest')
c           do 600 i=1,nmdata
c           write(12,*) zmdata(i),smdata(i),c1m(i),c2m(i),c3m(i)
c600       continue
c          close (12)
          zmdata(nmdata)=zmmax
          return
c correction to musq!! ze1=55.0 May 9,97
c
c     assign weights and smoothing parameter
c
      entry fllist(c0)
          wic=wi/(2.*pi)
          hstore=h*float(hcount)
          if(ylong) then
          write(iuout,9000)
 9000     format(
     *    'internal wave parameters'/)
      write(iuout,9100) na,n0b,jstar,zeta,gamad,lat,freq,hstore,c0,
     *wic,q0,msqmax,mj,njinv,avj
       endif
 9100 format(
     *'na        reference buoyancy frequency  ',g12.5,'cyc/hr    '/
     *'n0b       dispersion relation parameter ',g12.5,'m-cph     '/
     *'jstar     characteristic mode number    ',g12.5,'          '/
     *'zeta      rms int. wave displacement    ',g12.5,'          '/
     *'gamma     adiabatic gradient            ',g12.5,'          '/
     *'latitude                                ',g12.5,'deg       '/
     * /
     * 'acoustic frequency                     ',g12.5,'hz        '/
     * /
     *'CALCULATED PARAMETERS                                      '/
     * /
     *'storage step size                       ',g12.5,'m         '/
     *'reference sound speed                   ',g12.5,'m/sec     '/
     *'inertial frequency                      ',g12.5,'cyc/hr    '/
     *'acoustic wave number                    ',g12.5,'1/m       '/
     *'msqmax    maximum mu squared            ',g12.5,'          '/
     *'mj                                      ',g12.5,'          '/
     *'njinv                                   ',g12.5,'          '/
     *'avj        <1/j>                        ',g12.5,'          '/)
      return
      end
